(function() {
  var MutationObserver, __http_fetch, __http_preloads, __pagination_buffered, __pagination_current, __pagination_ifs, __pagination_targets, __pagination_transition_opacity, __ref_listeners, __ref_refs, __state_listeners, __state_mutation_listeners, __state_states, __timer_timers, dataFuns, eventMaps, formatBody, formatDataWord, helpers, preRegistered, propMaps, registerDataWord;

  preRegistered = [];

  eventMaps = {
    "double-click": "dblclick",
    "mouse-down": "mousedown",
    "mouse-up": "mouseup",
    "mouse-over": "mouseover",
    "mouse-out": "mouseout",
    "mouse-move": "mousemove",
    "mouse-enter": "mouseenter",
    "mouse-leave": "mouseleave",
    "key-down": "keydown",
    "key-up": "keyup",
    "key-press": "keypress"
  };

  propMaps = {
    "text": "textContent",
    "unsafe-html": "innerHTML",
    "value": "value"
  };

  dataFuns = {};

  helpers = {
    propExists: function(element, attribute) {
      return element.getAttribute(attribute) !== null;
    },
    matchParent: function(element, attribute) {
      var parent;
      parent = element.parentElement;
      while (parent) {
        if (helpers.propExists(parent, attribute)) {
          return parent;
        }
        parent = parent.parentElement;
      }
      return null;
    },
    sleep: function(ms) {
      return new Promise(function(resolve) {
        return setTimeout(resolve, ms);
      });
    }
  };

  formatDataWord = function(camelCase) {
    return "sn-" + camelCase.replace(/([A-Z])/g, function(match) {
      return `-${match.toLowerCase()}`;
    });
  };

  registerDataWord = function(name, func) {
    return dataFuns[name] = func;
  };

  registerDataWord("components", async function(element, attribute) {
    var html, path, response;
    path = attribute;
    response = (await fetch(path));
    if (response.ok) {
      html = (await response.text());
      return element.outerHTML = html;
    } else {
      throw new Error("Failed to fetch components from " + path);
    }
  });

  registerDataWord("copy", function(element, attribute) {
    var copyOn, el, expiresOn, fn, i, j, ref1, results, times;
    times = parseInt(attribute);
    copyOn = element.getAttribute(formatDataWord("copyOn")) || "now";
    expiresOn = helpers.propExists(element, formatDataWord("copyExpiresOn")) && element.getAttribute(formatDataWord("copyExpiresOn"));
    if (isNaN(times)) {
      throw new Error("Invalid number of times: " + attribute);
    }
    if (copyOn === "now") {
      results = [];
      for (i = j = 1, ref1 = times; (1 <= ref1 ? j <= ref1 : j >= ref1); i = 1 <= ref1 ? ++j : --j) {
        el = element.cloneNode(true);
        el.removeAttribute("sn-copy");
        results.push(element.insertAdjacentElement("afterend", el));
      }
      return results;
    } else {
      fn = function() {
        var k, ref2, results1;
        results1 = [];
        for (i = k = 1, ref2 = times; (1 <= ref2 ? k <= ref2 : k >= ref2); i = 1 <= ref2 ? ++k : --k) {
          el = element.cloneNode(true);
          el.removeAttribute("sn-copy");
          results1.push(element.insertAdjacentElement("afterend", el));
        }
        return results1;
      };
      element.addEventListener(eventMaps[copyOn] || copyOn, fn);
      if (expiresOn) {
        return element.addEventListener(eventMaps[expiresOn] || expiresOn, function() {
          return element.removeEventListener(eventMaps[copyOn] || copyOn, fn);
        });
      }
    }
  });

  registerDataWord("rm", function(element, attribute) {
    var before, fn, rmOn;
    rmOn = eventMaps[attribute] || attribute;
    before = element.getAttribute(formatDataWord("rmBefore")) || "";
    fn = function() {
      new Function(before).apply({
        element: element
      });
      return element.remove();
    };
    return element.addEventListener(rmOn, fn);
  });

  registerDataWord("eval", function(element, attribute) {
    var code;
    code = element.getAttribute(formatDataWord("eval"));
    return new Function(code).call({
      element: element
    });
  });

  __timer_timers = {};

  registerDataWord("timer-eval", function(element, attribute) {
    var code, interval;
    code = attribute;
    interval = element.getAttribute(formatDataWord("timerInterval")) || 1000;
    if (__timer_timers[element]) {
      clearInterval(__timer_timers[element]);
    }
    return __timer_timers[element] = setInterval(function() {
      if (!element) {
        return clearInterval(__timer_timers[element]);
      }
      return new Function(code).apply({
        element: element
      });
    }, parseInt(interval));
  });

  __http_preloads = {};

  __http_fetch = async function(url, method, type, data) {
    var response;
    response = null;
    if (method === "GET") {
      response = (await fetch(url, {
        method: method,
        headers: {
          "Accept": type
        }
      }));
    } else {
      response = (await fetch(url, {
        method: method,
        body: data
      }));
    }
    return response;
  };

  registerDataWord("get", async function(element, attribute) {
    var data, e, httpOn, preload, prop, response, result, set, type;
    type = element.getAttribute(formatDataWord("httpType")) || "text";
    set = propMaps[element.getAttribute(formatDataWord("httpSet"))] || "textContent";
    prop = element.getAttribute(formatDataWord("httpProp")) || "FULL_OBJECT";
    httpOn = element.getAttribute(formatDataWord("httpOn")) || "now";
    preload = helpers.propExists(element, formatDataWord("httpPreload"));
    data = "";
    if (preload && (!__http_preloads[attribute] || !__http_preloads[attribute].get)) {
      if (!__http_preloads[attribute]) {
        __http_preloads[attribute] = {};
      }
      response = (await __http_fetch(attribute, "GET", type, null));
      if (!response.ok) {
        data = "Error " + response.status;
        result = void 0;
        try {
          result = (await response.text());
        } catch (error) {
          e = error;
          result = "No further information available";
        }
        console.error("Error " + response.status + " - " + result);
      } else {
        if (type === "json") {
          data = (await response.json());
        } else {
          data = (await response.text());
        }
        __http_preloads[attribute].get = data;
      }
    }
    if (httpOn === "now") {
      if (!preload) {
        response = (await __http_fetch(attribute, "GET", type, null));
        if (!response.ok) {
          data = "Error " + response.status;
          result = void 0;
          try {
            result = (await response.text());
          } catch (error) {
            e = error;
            result = "No further information available";
          }
          console.error("Error " + response.status + " - " + result);
        } else {
          if (type === "json") {
            data = (await response.json());
          } else {
            data = (await response.text());
          }
        }
      } else {
        data = __http_preloads[attribute].get;
      }
      if (typeof data === "string") {
        return element[set] = data;
      } else if (typeof data === "object") {
        if (prop === "FULL_OBJECT") {
          return element[set] = JSON.stringify(data);
        } else {
          return element[set] = data[prop];
        }
      }
    } else {
      return element.addEventListener(eventMaps[httpOn] || httpOn, async function() {
        if (!preload) {
          response = (await __http_fetch(attribute, "GET", type, null));
          if (!response.ok) {
            data = "Error " + response.status;
            result = void 0;
            try {
              result = (await response.text());
            } catch (error) {
              e = error;
              result = "No further information available";
            }
            console.error("Error " + response.status + " - " + result);
          } else {
            if (type === "json") {
              data = (await response.json());
            } else {
              data = (await response.text());
            }
          }
        } else {
          data = __http_preloads[attribute].get;
        }
        if (typeof data === "string") {
          return element[set] = data;
        } else if (typeof data === "object") {
          if (prop === "FULL_OBJECT") {
            return element[set] = JSON.stringify(data);
          } else {
            return element[set] = data[prop];
          }
        }
      });
    }
  });

  formatBody = function(text) {
    text = text.replace(/\\n/g, "\n");
    text = text.replace(/\\t/g, "\t");
    text = text.replace(/state\{(.*?)\}/g, function(match, p1) {
      return __state_states[p1].element[propMaps[__state_states[p1].attribute]];
    });
    text = text.replace(/unsafeEval\{(.*?)\}/g, function(match, p1) {
      var e;
      e = new Function("return " + p1).apply({
        state: function(name) {
          return __state_states[name].element[propMaps[__state_states[name].attribute]];
        }
      });
      return e;
    });
    text = text.replace(/\{ref\{(.*?)}.(.*?)\}/g, function(match, p1, p2) {
      return __ref_refs[p1][p2];
    });
    return text;
  };

  registerDataWord("post", async function(element, attribute) {
    var body, data, e, httpOn, preload, prop, response, result, set, type;
    type = element.getAttribute(formatDataWord("httpType")) || "text";
    set = propMaps[element.getAttribute(formatDataWord("httpSet"))] || "textContent";
    prop = element.getAttribute(formatDataWord("httpProp")) || "FULL_OBJECT";
    httpOn = element.getAttribute(formatDataWord("httpOn")) || "now";
    preload = helpers.propExists(element, formatDataWord("httpPreload"));
    body = element.getAttribute(formatDataWord("httpBody")) || "";
    data = "";
    if (httpOn === "now") {
      if (!preload) {
        response = (await __http_fetch(attribute, "POST", type, formatBody(body)));
        if (!response.ok) {
          data = "Error " + response.status;
          result = void 0;
          try {
            result = (await response.text());
          } catch (error) {
            e = error;
            result = "No further information available";
          }
          console.error("Error " + response.status + " - " + result);
        } else {
          if (type === "json") {
            data = (await response.json());
          } else {
            data = (await response.text());
          }
        }
      } else {
        if (!__http_preloads[attribute] || !__http_preloads[attribute].post) {
          if (!__http_preloads[attribute]) {
            __http_preloads[attribute] = {};
          }
          response = (await __http_fetch(attribute, "POST", type, formatBody(body)));
          if (!response.ok) {
            data = "Error " + response.status;
            result = void 0;
            try {
              result = (await response.text());
            } catch (error) {
              e = error;
              result = "No further information available";
            }
            console.error("Error " + response.status + " - " + result);
          } else {
            if (type === "json") {
              data = (await response.json());
            } else {
              data = (await response.text());
            }
          }
          __http_preloads[attribute].post = data;
        }
        data = __http_preloads[attribute].post;
      }
      if (typeof data === "string") {
        return element[set] = data;
      } else if (typeof data === "object") {
        if (prop === "FULL_OBJECT") {
          return element[set] = JSON.stringify(data);
        } else {
          return element[set] = data[prop];
        }
      }
    } else {
      return element.addEventListener(eventMaps[httpOn] || httpOn, async function() {
        if (!preload) {
          response = (await __http_fetch(attribute, "POST", type, formatBody(body)));
          if (!response.ok) {
            data = "Error " + response.status;
            result = void 0;
            try {
              result = (await response.text());
            } catch (error) {
              e = error;
              result = "No further information available";
            }
            console.error("Error " + response.status + " - " + result);
          } else {
            if (type === "json") {
              data = (await response.json());
            } else {
              data = (await response.text());
            }
          }
        } else {
          if (!__http_preloads[attribute] || !__http_preloads[attribute].post) {
            if (!__http_preloads[attribute]) {
              __http_preloads[attribute] = {};
            }
            response = (await __http_fetch(attribute, "POST", type, formatBody(body)));
            if (!response.ok) {
              data = "Error " + response.status;
              result = void 0;
              try {
                result = (await response.text());
              } catch (error) {
                e = error;
                result = "No further information available";
              }
              console.error("Error " + response.status + " - " + result);
            } else {
              if (type === "json") {
                data = (await response.json());
              } else {
                data = (await response.text());
              }
            }
            __http_preloads[attribute].post = data;
          }
          data = __http_preloads[attribute].post;
        }
        if (typeof data === "string") {
          return element[set] = data;
        } else if (typeof data === "object") {
          if (prop === "FULL_OBJECT") {
            return element[set] = JSON.stringify(data);
          } else {
            return element[set] = data[prop];
          }
        }
      });
    }
  });

  __pagination_current = {};

  __pagination_targets = {};

  __pagination_ifs = {};

  __pagination_buffered = false;

  __pagination_transition_opacity = async function(element, reverse, speed) {
    var i, j, k, oldTransition, ref1, ref2, step;
    step = (speed || 1000) / 10;
    oldTransition = element.style.transition;
    element.style.transition = "none";
    if (reverse) {
      for (i = j = 0, ref1 = step; (0 <= ref1 ? j <= ref1 : j >= ref1); i = 0 <= ref1 ? ++j : --j) {
        element.style.opacity = 1 - (i / step);
        await helpers.sleep(1);
      }
    } else {
      for (i = k = 0, ref2 = step; (0 <= ref2 ? k <= ref2 : k >= ref2); i = 0 <= ref2 ? ++k : --k) {
        element.style.opacity = i / step;
        await helpers.sleep(1);
      }
    }
    return element.style.transition = oldTransition;
  };

  registerDataWord("pag-if", function(element, attribute) {
    var onDefault, onVisible, pagContext, targ;
    __pagination_buffered = true;
    pagContext = element.getAttribute(formatDataWord("pagCtx")) || "global";
    targ = document.querySelector('[' + formatDataWord('pagTarget') + '="' + attribute + '"][sn-pag-ctx="' + pagContext + '"]');
    onVisible = element.getAttribute(formatDataWord('pagOnVisible')) || "";
    onDefault = element.getAttribute(formatDataWord('pagDefault')) || "";
    if (!__pagination_ifs[pagContext]) {
      __pagination_ifs[pagContext] = {};
    }
    if (!__pagination_ifs[pagContext][attribute]) {
      __pagination_ifs[pagContext][attribute] = [];
    }
    __pagination_ifs[pagContext][attribute].push({
      element: element,
      target: targ,
      onVisible: onVisible,
      onDefault: onDefault
    });
    return __pagination_buffered = false;
  });

  registerDataWord("pag-to", async function(element, attribute) {
    var anim, j, k, len, len1, onVisible, pagContext, pagIf, pagOn, ref1, ref2, targ;
    pagOn = element.getAttribute(formatDataWord("pagOn")) || "click";
    pagContext = element.getAttribute(formatDataWord("pagCtx")) || "global";
    targ = document.querySelector('[' + formatDataWord('pagTarget') + '="' + attribute + '"][sn-pag-ctx="' + pagContext + '"]');
    onVisible = targ.getAttribute(formatDataWord('pagOnVisible')) || "";
    anim = helpers.propExists(targ, formatDataWord('pagAnim')) && targ.getAttribute(formatDataWord('pagAnim')) !== "false" || false;
    while (__pagination_buffered) {
      await helpers.sleep(100);
    }
    if (targ) {
      targ.__OLD_DISPLAY = targ.style.display;
      if (localStorage.getItem('sn-pag-' + pagContext) === attribute) {
        __pagination_current[pagContext] = targ;
        targ.style.display = targ.__OLD_DISPLAY || "block";
        targ.setAttribute('sn-pag-current', 'true');
        new Function(onVisible).apply({
          element: targ,
          caller: element
        });
        if (!__pagination_ifs[pagContext]) {
          __pagination_ifs[pagContext] = {};
        }
        if (!__pagination_ifs[pagContext][attribute]) {
          __pagination_ifs[pagContext][attribute] = [];
        }
        ref1 = __pagination_ifs[pagContext][attribute];
        for (j = 0, len = ref1.length; j < len; j++) {
          pagIf = ref1[j];
          if (pagIf.onVisible) {
            new Function(pagIf.onVisible).apply({
              element: pagIf.element,
              caller: element
            });
          }
        }
      } else if (!__pagination_current[pagContext] && helpers.propExists(targ, formatDataWord('pagDefault')) && !localStorage.getItem('sn-pag-' + pagContext)) {
        if (!localStorage.getItem('sn-pag-' + pagContext)) {
          localStorage.setItem('sn-pag-' + pagContext, attribute);
        }
        __pagination_current[pagContext] = targ;
        targ.style.display = targ.__OLD_DISPLAY || "block";
        targ.setAttribute('sn-pag-current', 'true');
        new Function(onVisible).apply({
          element: targ,
          caller: element
        });
        if (!__pagination_ifs[pagContext]) {
          __pagination_ifs[pagContext] = {};
        }
        if (!__pagination_ifs[pagContext][attribute]) {
          __pagination_ifs[pagContext][attribute] = [];
        }
        ref2 = __pagination_ifs[pagContext][attribute];
        for (k = 0, len1 = ref2.length; k < len1; k++) {
          pagIf = ref2[k];
          if (pagIf.onVisible) {
            new Function(pagIf.onVisible).apply({
              element: pagIf.element,
              caller: element
            });
          }
        }
      } else {
        targ.style.display = "none";
        targ.removeAttribute('sn-pag-current');
      }
      if (!__pagination_targets[pagContext]) {
        __pagination_targets[pagContext] = [];
      }
      __pagination_targets[pagContext].push({
        attribute: attribute,
        target: targ,
        animated: anim
      });
      return element.addEventListener(eventMaps[pagOn] || pagOn, async function() {
        var l, len2, len3, len4, len5, m, n, o, ref3, ref4, ref5, ref6, target;
        if (localStorage.getItem('sn-pag-' + pagContext) !== attribute) {
          localStorage.setItem('sn-pag-' + pagContext, attribute);
        }
        new Function(onVisible).apply({
          element: targ,
          caller: element
        });
        if (!__pagination_ifs[pagContext]) {
          __pagination_ifs[pagContext] = {};
        }
        if (!__pagination_ifs[pagContext][attribute]) {
          __pagination_ifs[pagContext][attribute] = [];
        }
        ref3 = Object.entries(__pagination_ifs[pagContext]);
        for (l = 0, len2 = ref3.length; l < len2; l++) {
          pagIf = ref3[l];
          if (pagIf[0] !== attribute) {
            ref4 = pagIf[1];
            for (m = 0, len3 = ref4.length; m < len3; m++) {
              pagIf = ref4[m];
              if (pagIf.onDefault) {
                new Function(pagIf.onDefault).apply({
                  element: pagIf.element,
                  caller: element
                });
              }
            }
          }
        }
        ref5 = __pagination_ifs[pagContext][attribute];
        for (n = 0, len4 = ref5.length; n < len4; n++) {
          pagIf = ref5[n];
          if (pagIf.onVisible) {
            new Function(pagIf.onVisible).apply({
              element: pagIf.element,
              caller: element
            });
          }
        }
        ref6 = __pagination_targets[pagContext];
        for (o = 0, len5 = ref6.length; o < len5; o++) {
          target = ref6[o];
          if (target.target !== targ) {
            if (target.animated) {
              await __pagination_transition_opacity(target.target, true, target.target.getAttribute(formatDataWord('pagAnim')) || 1000);
            }
            target.target.style.display = "none";
            target.target.removeAttribute('sn-pag-current');
            if (!__pagination_ifs[pagContext]) {
              __pagination_ifs[pagContext] = {};
            }
            if (!__pagination_ifs[pagContext][target.attribute]) {
              __pagination_ifs[pagContext][target.attribute] = [];
            }
          }
        }
        targ.style.display = targ.__OLD_DISPLAY || "block";
        targ.setAttribute('sn-pag-current', 'true');
        if (anim) {
          return (await __pagination_transition_opacity(targ, false, targ.getAttribute(formatDataWord('pagAnim')) || 1000));
        }
      });
    } else {
      return console.error('No pagination target found for [' + formatDataWord('pag-target') + '="' + attribute + '"]');
    }
  });

  registerDataWord("pag-parent", function(element, attribute) {
    var parent, result, results, set, withDefault;
    set = propMaps[attribute] || "textContent";
    withDefault = helpers.propExists(element, formatDataWord('pagDefault'));
    parent = element.parentElement;
    results = [];
    while (parent) {
      if (helpers.propExists(parent, formatDataWord('pagTarget'))) {
        result = parent.getAttribute(formatDataWord('pagTarget'));
        if (withDefault) {
          if (helpers.propExists(parent, formatDataWord('pagDefault'))) {
            result = "default";
          } else {
            result = "customized";
          }
        }
        element[set] = result;
        break;
      }
      results.push(parent = parent.parentElement);
    }
    return results;
  });

  __ref_refs = {};

  __ref_listeners = {};

  registerDataWord("ref", function(element, attribute) {
    var name;
    name = attribute;
    element.__DATA_REF_NAME__ = name;
    if (__ref_refs[name]) {
      return console.error("Ref " + name + " already exists");
    } else {
      __ref_refs[name] = element;
      return element.addEventListener("remove", function() {
        return delete __ref_refs[name];
      });
    }
  });

  registerDataWord("ref-listen", function(element, attribute) {
    var defaultUpdate, name, onUpdate;
    onUpdate = element.getAttribute(formatDataWord("refOnUpdate")) || "this.element.textContent = this.ref.textContent";
    defaultUpdate = element.getAttribute(formatDataWord("refListenDefault")) || "this.element.textContent = '...'";
    name = attribute;
    if (!__ref_refs[name]) {
      throw new Error("Ref not found: " + name);
    }
    __ref_refs[name].addEventListener("remove", function() {
      return new Function(defaultUpdate).apply({
        element: element,
        ref: __ref_refs[name]
      });
    });
    new Function(defaultUpdate).apply({
      element: element,
      ref: __ref_refs[name]
    });
    if (!__ref_listeners[name]) {
      __ref_listeners[name] = [];
    }
    if (Object.values(__ref_listeners).find(function(listener) {
      return listener.element === element;
    })) {
      throw new Error("Element already has a ref listener: " + element);
    }
    return __ref_listeners[name].push({
      element: element,
      onUpdate: onUpdate,
      defaultUpdate: defaultUpdate
    });
  });

  registerDataWord("copy-ref", function(element, attribute) {
    var copyOn, el, expiresOn, fn, i, j, ref, ref1, refElement, results, times;
    times = parseInt(element.getAttribute(formatDataWord("copyTimes"))) || 1;
    copyOn = element.getAttribute(formatDataWord("copyOn")) || "now";
    expiresOn = helpers.propExists(element, formatDataWord("copyExpiresOn")) && element.getAttribute(formatDataWord("copyExpiresOn"));
    if (isNaN(times)) {
      throw new Error("Invalid number of times: " + attribute);
    }
    ref = attribute;
    if (!ref) {
      throw new Error("No ref specified");
    }
    refElement = __ref_refs[ref];
    if (!refElement) {
      throw new Error("Ref not found: " + ref);
    }
    if (copyOn === "now") {
      results = [];
      for (i = j = 1, ref1 = times; (1 <= ref1 ? j <= ref1 : j >= ref1); i = 1 <= ref1 ? ++j : --j) {
        el = refElement.cloneNode(true);
        el.removeAttribute("sn-copy-ref");
        results.push(element.insertAdjacentElement("afterend", el));
      }
      return results;
    } else {
      fn = function() {
        var k, ref2, results1;
        results1 = [];
        for (i = k = 1, ref2 = times; (1 <= ref2 ? k <= ref2 : k >= ref2); i = 1 <= ref2 ? ++k : --k) {
          el = refElement.cloneNode(true);
          el.removeAttribute("sn-copy-ref");
          results1.push(element.insertAdjacentElement("afterend", el));
        }
        return results1;
      };
      element.addEventListener(eventMaps[copyOn] || copyOn, fn);
      if (expiresOn) {
        return element.addEventListener(eventMaps[expiresOn] || expiresOn, function() {
          return element.removeEventListener(eventMaps[copyOn] || copyOn, fn);
        });
      }
    }
  });

  registerDataWord("rm-ref", function(element, attribute) {
    var before, expiresOn, fn, ref, rmOn;
    rmOn = eventMaps[attribute] || attribute;
    ref = element.getAttribute(formatDataWord("rmTarget")) || "";
    before = element.getAttribute(formatDataWord("rmBefore")) || "";
    expiresOn = helpers.propExists(element, formatDataWord("rmExpiresOn")) && element.getAttribute(formatDataWord("rmExpiresOn"));
    fn = function() {
      new Function(before).apply({
        ref: __ref_refs[ref],
        element: element
      });
      __ref_refs[ref].remove();
      delete __ref_refs[ref];
      return (__ref_listeners[ref] || []).forEach(function(listener) {
        return new Function(listener.defaultUpdate).apply({
          element: listener.element,
          ref: __ref_refs[ref]
        });
      });
    };
    element.addEventListener(rmOn, fn);
    if (expiresOn) {
      return element.addEventListener(eventMaps[expiresOn] || expiresOn, function() {
        return element.removeEventListener(rmOn, fn);
      });
    }
  });

  __state_states = {};

  __state_listeners = {};

  __state_mutation_listeners = [];

  registerDataWord("state", function(element, attribute) {
    var prop;
    prop = element.getAttribute(formatDataWord("stateProp")) || (function() {
      throw new Error("No stateProp defined for state: " + attribute);
    })();
    if (Object.values(__state_states).find(function(state) {
      return state.element === element;
    })) {
      throw new Error("Element already has a state: " + element);
    }
    if (!__state_states[attribute]) {
      __state_states[attribute] = {};
    } else {
      throw new Error("State already exists: " + attribute);
    }
    if (!__state_listeners[attribute]) {
      __state_listeners[attribute] = [];
    }
    element.__DATA_STATE_NAME__ = attribute;
    __state_states[attribute].attribute = prop;
    __state_states[attribute].element = element;
    __state_mutation_listeners.push({
      element: element,
      attribute: attribute,
      set: prop
    });
    return element.addEventListener("remove", function() {
      delete __state_states[attribute];
      return delete __state_listeners[attribute];
    });
  });

  registerDataWord("state-listen", function(element, attribute) {
    var set;
    set = propMaps[element.getAttribute(formatDataWord("stateListenSet"))] || "text";
    if (!__state_states[attribute]) {
      throw new Error("State not found: " + attribute);
    }
    if (!__state_listeners[attribute]) {
      __state_listeners[attribute] = [];
    }
    if (Object.values(__state_listeners).find(function(listener) {
      return listener.element === element;
    })) {
      throw new Error("Element already has a state listener: " + element);
    }
    __state_listeners[attribute].push({
      element: element,
      set: set
    });
    if (__state_states[attribute].element[set]) {
      return element[set] = __state_states[attribute].element[set] || "...";
    }
  });

  registerDataWord("reflect-states", function(element, attribute) {
    var j, len, ref1, set, state, stateNames;
    set = propMaps[attribute] || "textContent";
    stateNames = [];
    __state_states = new Proxy(__state_states, {
      set: function(target, prop, value) {
        if (!__state_states[prop]) {
          target[prop] = value;
          stateNames.push(prop);
          element[set] = stateNames.join(", ");
        }
        return true;
      }
    });
    ref1 = Object.entries(__state_states);
    for (j = 0, len = ref1.length; j < len; j++) {
      state = ref1[j];
      stateNames.push(state[0]);
    }
    return element[set] = stateNames.join(", ");
  });

  MutationObserver = window.MutationObserver || window.WebKitMutationObserver || window.MozMutationObserver;

  document.addEventListener('DOMContentLoaded', async function() {
    var __states_observer, el, func, j, len, name, withName;
    for (name in dataFuns) {
      func = dataFuns[name];
      withName = [...document.querySelectorAll(`[sn-${name}]`)];
      for (j = 0, len = withName.length; j < len; j++) {
        el = withName[j];
        if (helpers.matchParent(el, "sn-ignore-children")) {
          continue;
        }
        await func(el, el.getAttribute(`sn-${name}`));
        if (!preRegistered.map(function(e) {
          return e.element;
        }).includes(el)) {
          preRegistered.push({
            element: el,
            attrs: []
          });
        }
        preRegistered.find(function(e) {
          return e.element === el;
        }).attrs.push(`sn-${name}`);
      }
    }
    __states_observer = new MutationObserver(async function(mutations) {
      var a, element, k, l, len1, len2, len3, len4, len5, listener, m, mutation, n, o, r, ref1, ref2, ref3, ref4, results;
      results = [];
      for (k = 0, len1 = mutations.length; k < len1; k++) {
        mutation = mutations[k];
        ref1 = Object.entries(__state_states);
        for (l = 0, len2 = ref1.length; l < len2; l++) {
          listener = ref1[l];
          if (listener[0] !== mutation.target.__DATA_STATE_NAME__) {
            continue;
          }
          if (__state_listeners[listener[0]]) {
            ref2 = __state_listeners[listener[0]];
            for (m = 0, len3 = ref2.length; m < len3; m++) {
              element = ref2[m];
              a = __state_states[listener[0]].attribute;
              r = mutation.target[propMaps[a] || a];
              element.element[element.set] = r;
            }
          }
        }
        ref3 = Object.entries(__ref_listeners);
        for (n = 0, len4 = ref3.length; n < len4; n++) {
          listener = ref3[n];
          if (listener[0] !== mutation.target.__DATA_REF_NAME__) {
            continue;
          }
          __ref_refs[listener[0]] = mutation.target;
          if (__ref_listeners[listener[0]]) {
            ref4 = __ref_listeners[listener[0]];
            for (o = 0, len5 = ref4.length; o < len5; o++) {
              element = ref4[o];
              new Function(element.onUpdate).apply({
                element: element.element,
                ref: __ref_refs[listener[0]]
              });
            }
          }
        }
        if (!helpers.matchParent(mutation.target, "sn-ignore-children") && mutation.type === "childList" && !preRegistered.map(function(e) {
          return e.element;
        }).includes(mutation.target)) {
          results.push((await (async function() {
            var results1;
            results1 = [];
            for (name in dataFuns) {
              func = dataFuns[name];
              if (helpers.propExists(mutation.target, `sn-${name}`)) {
                await func(mutation.target, mutation.target.getAttribute(`sn-${name}`));
                if (!preRegistered.map(function(e) {
                  return e.element;
                }).includes(mutation.target)) {
                  preRegistered.push({
                    element: mutation.target,
                    attrs: []
                  });
                }
                results1.push(preRegistered.find(function(e) {
                  return e.element === mutation.target;
                }).attrs.push(`sn-${name}`));
              } else {
                results1.push(void 0);
              }
            }
            return results1;
          })()));
        } else if (mutation.type === "attributes" && mutation.attributeName.startsWith("sn-")) {
          if (preRegistered.find(function(e) {
            return e.element === mutation.target;
          })) {
            results.push((await (async function() {
              var len6, p, ref5, results1;
              ref5 = preRegistered.find(function(e) {
                return e.element === mutation.target;
              }).attrs;
              results1 = [];
              for (p = 0, len6 = ref5.length; p < len6; p++) {
                name = ref5[p];
                if (name === mutation.attributeName) {
                  continue;
                }
                if (dataFuns[name]) {
                  results1.push((await dataFuns[name](mutation.target, mutation.target.getAttribute(`sn-${name}`))));
                } else {
                  results1.push(void 0);
                }
              }
              return results1;
            })()));
          } else {
            results.push(void 0);
          }
        } else {
          results.push(void 0);
        }
      }
      return results;
    });
    return __states_observer.observe(document, {
      attributes: true,
      childList: true,
      subtree: true
    });
  });

}).call(this);
